-
Notifications
You must be signed in to change notification settings - Fork 202
Expand file tree
/
Copy pathcuda_utils.cu
More file actions
76 lines (58 loc) · 2 KB
/
Copy pathcuda_utils.cu
File metadata and controls
76 lines (58 loc) · 2 KB
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
/*
Copyright 2016 Fixstars Corporation
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http ://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
#include "internal.h"
#include "cuda_to_hip.h"
#include "host_utility.h"
namespace
{
__global__ void cast_16bit_8bit_array_kernel(const uint16_t* arr16bits, uint8_t* arr8bits, int num_elements)
{
const int i = blockIdx.x * blockDim.x + threadIdx.x;
if (i < num_elements)
arr8bits[i] = static_cast<uint8_t>(arr16bits[i]);
}
__global__ void cast_8bit_16bit_array_kernel(const uint8_t* arr8bits, uint16_t* arr16bits, int num_elements)
{
const int i = blockIdx.x * blockDim.x + threadIdx.x;
if (i < num_elements)
arr16bits[i] = static_cast<uint16_t>(arr8bits[i]);
}
} // namespace
namespace sgm
{
namespace details
{
void cast_16bit_to_8bit(const DeviceImage& src, DeviceImage& dst)
{
const int w = src.cols;
const int h = src.rows;
dst.create(h, w, SGM_8U, src.step);
const int num_elements = h * src.step;
const int block = 1024;
const int grid = divUp(num_elements, block);
cast_16bit_8bit_array_kernel<<<grid, block>>>(src.ptr<uint16_t>(), dst.ptr<uint8_t>(), num_elements);
CUDA_CHECK(cudaGetLastError());
}
void cast_8bit_to_16bit(const DeviceImage& src, DeviceImage& dst)
{
const int w = src.cols;
const int h = src.rows;
dst.create(h, w, SGM_16U, src.step);
const int num_elements = h * src.step;
const int block = 1024;
const int grid = divUp(num_elements, block);
cast_8bit_16bit_array_kernel<<<grid, block>>>(src.ptr<uint8_t>(), dst.ptr<uint16_t>(), num_elements);
CUDA_CHECK(cudaGetLastError());
}
} // namespace details
} // namespace sgm